#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Config_1 = require("./utils/Config");
const SubscriberManager_1 = require("./utils/SubscriberManager");
const fs_1 = require("fs");
const ShowWelcomeMessageOnStartup_1 = require("./services/ShowWelcomeMessageOnStartup");
const DemoRecordingHelper_1 = require("./services/DemoRecordingHelper");
const DemoPlaybackHelper_1 = require("./services/DemoPlaybackHelper");
const LogHelper_1 = require("./utils/LogHelper");
const ShowHelpMessageWhenAsked_1 = require("./services/ShowHelpMessageWhenAsked");
const path_1 = require("path");
const log = LogHelper_1.LogHelper.getLogger('Main');
(async () => {
    LogHelper_1.LogHelper.configure();
    if (!fs_1.existsSync(path_1.join(Config_1.Config.getConfig().csgo.csgo_demos_folder, "..", "csgo.exe"))) {
        log.fatal(`Couldn't find CS:GO's executable at the path '${path_1.join(Config_1.Config.getConfig().csgo.csgo_demos_folder, "..", "csgo.exe")}'.`);
        log.fatal(`This means that DemoRecordingHelper will be unable to locate the demos folder to check for name conflicts.`);
        throw Error('csgo_demos_folder in config.ini is misconfigured.');
    }
    SubscriberManager_1.SubscriberManager.subscribe(new ShowWelcomeMessageOnStartup_1.ShowWelcomeMessageOnStartup());
    SubscriberManager_1.SubscriberManager.subscribe(new ShowHelpMessageWhenAsked_1.ShowHelpMessageWhenAsked());
    SubscriberManager_1.SubscriberManager.subscribe(new DemoRecordingHelper_1.DemoRecordingHelper());
    SubscriberManager_1.SubscriberManager.subscribe(new DemoPlaybackHelper_1.DemoPlaybackHelper());
    await SubscriberManager_1.SubscriberManager.init();
    SubscriberManager_1.SubscriberManager.begin().then();
})().catch(reason => {
    log.fatal(reason);
    log.fatal('Exited due to a fatal error. Please see above for details.');
});
//# sourceMappingURL=../src/Main.js.map