"use strict";
/*
 * See the top comment of DemoRecordingHelper for an explanation of what this class is responsible for.
 *
 * NOTE: YOU NO LONGER NEED TO HAVE THIS RUNNING WHEN PERFORMING DEMO PLAYBACK UNLESS THE DEMO WAS RECORDED
 * WITH A VERSION EARLIER THAN 1.1.0!!! You'll know if you need this running if you
 * notice yourself talking during demo playback but don't hear any audio.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.DemoPlaybackHelper = void 0;
const Config_1 = require("../utils/Config");
const VoicePlayerVolume_1 = require("../utils/VoicePlayerVolume");
const LogHelper_1 = require("../utils/LogHelper");
const SubscriberManager_1 = require("../utils/SubscriberManager");
const DemoRecordingHelper_1 = require("./DemoRecordingHelper");
/**
 * DemoPlaybackHelper is responsible for listening to the console for messages left behind by DemoRecordingHelper.
 * For instance, a message about a player being muted might have been left behind and DemoPlaybackHelper acts on this
 * information by unmuting the player. This is just an example of its purpose and isn't actually necessary any more
 * with demos recorded after the changes made in 1.1.0.
 */
class DemoPlaybackHelper {
    name() {
        return DemoPlaybackHelper.name;
    }
    canHandle(consoleLine) {
        return !DemoRecordingHelper_1.DemoRecordingHelper.synchronouslyCheckIfRecording() &&
            DemoPlaybackHelper.playerMutedByDemoHelperRegExp.test(consoleLine);
    }
    async handleLine(consoleLine) {
        if (await DemoPlaybackHelper.currentlyPlayingADemo()) {
            if (DemoPlaybackHelper.playerMutedByDemoHelperRegExp.test(consoleLine)) {
                if (Config_1.Config.getConfig().demo_playback_helper.playback_voice_player_volume === "1") {
                    const match = DemoPlaybackHelper.playerMutedByDemoHelperRegExp.exec(consoleLine);
                    if (!match)
                        throw Error('Got null match when determining player name from the message left behind in the demo.');
                    const playerName = match[1];
                    //TODO: Additional testing required to make sure this doesn't fire before the game is ready to deal with it
                    DemoPlaybackHelper.log.info(`DemoPlaybackHelper found a line indicating DemoHelper muted ${playerName} so it unmuted them.`);
                    await VoicePlayerVolume_1.VoicePlayerVolume.setVoicePlayerVolumeByName(playerName, 1);
                }
                else {
                    DemoPlaybackHelper.log.info("DemoPlaybackHelper found a line indicating DemoHelper muted a player but demo_playback_helper.playback_voice_player_volume was set to 0 in the config file.");
                }
            }
        }
    }
}
exports.DemoPlaybackHelper = DemoPlaybackHelper;
DemoPlaybackHelper.log = LogHelper_1.LogHelper.getLogger('DemoPlaybackHelper');
// private static readonly demoPlaybackRegExp = RegExp('^Playing demo from (.*)\\.dem\\.$');
//A message like this will be echoed right when the demo starts recording.
DemoPlaybackHelper.playerMutedByDemoHelperRegExp = RegExp('^DemoHelper set the volume of player (.*) to 0\\.$');
DemoPlaybackHelper.demoInfoRegExp = RegExp('(Error - Not currently playing back a demo\\.)|(Demo contents for (.*)\\.dem:)');
DemoPlaybackHelper.currentlyPlayingADemo = async () => {
    const demoName = await DemoPlaybackHelper.getCurrentDemoName();
    return demoName.length > 0;
};
DemoPlaybackHelper.getCurrentDemoName = async () => {
    const consoleLine = await SubscriberManager_1.SubscriberManager.searchForValue('demo_info', DemoPlaybackHelper.demoInfoRegExp, false);
    const match = DemoPlaybackHelper.demoInfoRegExp.exec(consoleLine);
    if (!match)
        throw Error(`Failed to execute RegExp on the console's response to demo_info.`);
    return match[3] ? match[3] : '';
};
//# sourceMappingURL=../../src/services/DemoPlaybackHelper.js.map