"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Config = void 0;
const ini_1 = require("ini");
const fs_1 = require("fs");
const log4js_1 = require("log4js");
const path_1 = require("path");
class Config {
    /**
     * DO NOT CALL THIS METHOD EVER. THIS IS FOR INTERNAL USE INSIDE Config.ts ONLY!!!!!
     */
    static _initialize() {
        if (!fs_1.existsSync(Config.config_path)) {
            // noinspection SpellCheckingInspection
            log4js_1.configure({
                appenders: {
                    out: { type: 'stdout' },
                    app: { type: 'file', filename: 'application.log' }
                },
                categories: { default: { appenders: ['out', 'app'], level: 'info' } }
            });
            const log = log4js_1.getLogger('Config');
            log.fatal(`Couldn't find 'config.ini' expected at path '${Config.config_path}'.`);
            log.info(`It seems you haven't made a config.ini file yet. I'll make one for you now...`);
            fs_1.copyFileSync(Config.config_template_path, Config.config_path, fs_1.constants.COPYFILE_EXCL);
            log.info(`Created 'config.ini' at path ${Config.config_path}`);
            log.info(`Please modify config.ini to match your desired settings and then launch this program again.`);
            process.exit();
            // throw Error(`config.ini not found at path '${Config.config_path}'`);
        }
        Config.config = ini_1.parse(fs_1.readFileSync(Config.config_path, 'utf-8'));
    }
}
exports.Config = Config;
Config.config_path = path_1.join(__dirname, "..", "..", "config.ini");
Config.config_template_path = path_1.join(__dirname, "..", "..", "config.template.ini");
Config.getConfig = () => {
    return Config.config;
};
/*
 * This runs the _initialize method as soon as this class is declared The reason for doing this (and for configuring
 * a logger separately from LogHelper) is to be able to properly alert the user that they haven't configured
 * their config.ini file before other classes that statically access Config.getConfig() run into errors.
 *
 * This fails fast and lets the error thrown propagate to the top and exit before any errors arise from getting
 * properties from an undefined dictionary occur. I found this handy-dandy solution at https://stackoverflow.com/a/57097362/1687436
 */
try {
    Config._initialize();
}
catch (e) {
    throw e;
    // process.kill(process.pid, 'SIGTERM');
}
//# sourceMappingURL=../../src/utils/Config.js.map