"use strict";
/*
 * When I record POV demos, I like to keep them organized by naming them by game mode, map, and date. If I've already
 * played the same map and game mode as a previous demo, I append "-2" or a greater number to the end of the demo name
 * to indicate it was the second time playing that combination of game mode and map that day. If I have to disconnect
 * and stop recording the demo, I typically append "-pt2" to indicate this is the second part of the same demo.
 * This tool helps by automating the naming process including checking for existing demos.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.DemoNamingHelper = void 0;
const Cvars_1 = require("./Cvars");
const SubscriberManager_1 = require("./SubscriberManager");
const LogHelper_1 = require("./LogHelper");
class DemoNamingHelper {
}
exports.DemoNamingHelper = DemoNamingHelper;
/*
 * Notes: game_mode 0 and game_type 4 (known as "cooperative") is also the combination for guardian
 */
// noinspection SpellCheckingInspection
DemoNamingHelper.gameModeStrings = [
    ["casual", "armsrace", "training", "custom", "cooperative", "skirmish"],
    ["competitive", "demolition"],
    ["wingman", "deathmatch"]
];
DemoNamingHelper.log = LogHelper_1.LogHelper.getLogger('DemoNamingHelper');
DemoNamingHelper.mapFromStatusRegExp = RegExp('map\\s+: ([a-zA-Z_]+).*');
/**
 * See https://totalcsgo.com/command/gametype
 * @returns {Promise<string>} the human friendly summarization of what
 * game mode is being played. This ignores the case of Scrim Competitive 5v5
 * and will always refer to Scrim Competitive 5v5 and 2v2 as "wingman".
 */
DemoNamingHelper.getGameModeString = async () => {
    let gameMode, gameType;
    try {
        gameMode = Number(await Cvars_1.Cvars.getCvar('game_mode'));
        gameType = Number(await Cvars_1.Cvars.getCvar('game_type'));
    }
    catch (e) {
        throw e;
    }
    return DemoNamingHelper.gameModeStrings[gameMode][gameType];
};
DemoNamingHelper.makeTimestamp = () => {
    const d = new Date();
    return `${d.getMonth() + 1}-${d.getDate()}-${d.getFullYear()}`;
};
DemoNamingHelper.getMapName = async (excludePrefix) => {
    const mapLine = await SubscriberManager_1.SubscriberManager.searchForValue('status', DemoNamingHelper.mapFromStatusRegExp, false);
    const mapName = DemoNamingHelper.mapFromStatusRegExp.exec(mapLine);
    if (mapName) {
        if (excludePrefix) {
            //Attempt to remove the first underscore in the map name and everything before it
            return mapName[1].substr(mapName[1].indexOf('_') + 1);
        }
        else {
            return mapName[1];
        }
    }
    else {
        DemoNamingHelper.log.error('Failed to match regex when looking for map name.');
        return 'UNKNOWN';
    }
};
//# sourceMappingURL=../../src/utils/DemoNamingHelper.js.map