"use strict";
/*
 * While building these utilities I quickly realized that having tons of different functions vying for control
 * of the socket simply wasn't viable. I needed a stupid-simple way for a single authority to hold the socket connection
 * and manage it responsibly, passing text output to whoever needed it.
 */
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SubscriberManager = void 0;
const readline_1 = require("readline");
const net_1 = require("net");
const uuid_1 = require("uuid");
const LogHelper_1 = require("./LogHelper");
const Config_1 = require("./Config");
const TimeoutPromise_1 = require("./TimeoutPromise");
const pDefer = require("p-defer");
const waitOn = require("wait-on");
class SubscriberManager {
}
exports.SubscriberManager = SubscriberManager;
SubscriberManager.cvarEchoRegExp = RegExp('^\"([a-zA-Z_]+)\" = \"(\\d+)\".*');
SubscriberManager.config = Config_1.Config.getConfig();
SubscriberManager.port = SubscriberManager.config.csgo.netcon_port;
SubscriberManager.log = LogHelper_1.LogHelper.getLogger('SubscriberManager');
SubscriberManager.subscriberLog = LogHelper_1.LogHelper.getLogger('SubscriberManager.subscribers');
SubscriberManager.cvarSubscribersLog = LogHelper_1.LogHelper.getLogger('SubscriberManager.cvarSubscribers');
SubscriberManager.valueListenersLog = LogHelper_1.LogHelper.getLogger('SubscriberManager.valueListeners');
//For use with functions that are expecting the game not to be open yet
SubscriberManager.waitOnOpts = {
    resources: [
        `tcp:${SubscriberManager.port}`,
    ],
    interval: 100,
    timeout: 90000,
    // tcpTimeout: 1000, // tcp timeout in ms, default 300ms
    window: 500,
};
//For use with all other functions
SubscriberManager.waitOnOptsImpatient = {
    resources: [
        `tcp:${SubscriberManager.port}`,
    ],
    interval: 50,
    timeout: 1000,
    // tcpTimeout: 1000, // tcp timeout in ms, default 300ms
    window: 500,
};
/** These subscribers are expecting one or more lines of text, unprompted. They're here to stay for the most part. */
SubscriberManager.subscribers = [];
/** These subscribers are expecting a single line of output as soon as possible (like a cvar value) */
SubscriberManager.subscribedCvarValues = [];
/**
 * These subscribers act much like the cvar subscribers and are removed once dealt with. A regular expression is
 * used to determine if the given console output is meant for the temporary subscriber. If there's a match,
 * the subscriber is removed and the promise is fulfilled with the line output that matched the RegEx.
 */
SubscriberManager.specialOutputGrabbers = [];
SubscriberManager.init = async () => {
    SubscriberManager.log.info('Starting up...');
    SubscriberManager.log.debug(`Waiting to connect to CS:GO's netcon on port ${SubscriberManager.port}.`);
    try {
        await SubscriberManager.patientlyWaitForConsoleSocket();
    }
    catch (e) {
        throw e;
    }
    SubscriberManager.socket = net_1.connect(SubscriberManager.port, '127.0.0.1');
    SubscriberManager.log.debug(`Connected on port ${SubscriberManager.port}.`);
    SubscriberManager.socket.setEncoding('utf8');
    SubscriberManager.reader = readline_1.createInterface({
        input: SubscriberManager.socket,
        crlfDelay: Infinity
    });
    SubscriberManager.log.info('Ready!');
};
SubscriberManager.begin = async () => {
    var e_1, _a;
    try {
        for (var _b = __asyncValues(SubscriberManager.reader), _c; _c = await _b.next(), !_c.done;) {
            const rawLine = _c.value;
            const line = rawLine.trimEnd();
            let lineHandledBySpecialOutputGrabber = false;
            for (let i = 0; i < SubscriberManager.specialOutputGrabbers.length; i++) {
                if (SubscriberManager.specialOutputGrabbers[i][0].test(line)) {
                    SubscriberManager.valueListenersLog.trace(`Selected output grabber '${SubscriberManager.specialOutputGrabbers[i]}' to handle line '${line}'.`);
                    SubscriberManager.specialOutputGrabbers[i][1].resolve(line);
                    SubscriberManager.specialOutputGrabbers.splice(i, 1);
                    lineHandledBySpecialOutputGrabber = true;
                    break;
                }
            }
            if (lineHandledBySpecialOutputGrabber) {
                //This line has been processed and passing to other handlers could cause undefined behavior
                //Don't bother handing the line value to anyone else. Await the next line of console output
                continue;
            }
            const lineIsACvarValue = SubscriberManager.cvarEchoRegExp.test(line);
            let lineHandledByCvarListener = false;
            let cvarName, cvarValue;
            if (lineIsACvarValue) {
                const cvarOutput = SubscriberManager.cvarEchoRegExp.exec(line);
                if (cvarOutput) {
                    cvarName = cvarOutput[1];
                    cvarValue = cvarOutput[2];
                }
                else {
                    SubscriberManager.cvarSubscribersLog.error(`Cvar RegEX matched output for a cvar but couldn't properly capture the content. Console output was '${line}'.`);
                    continue;
                }
            }
            for (let i = 0; i < SubscriberManager.subscribedCvarValues.length; i++) {
                if (SubscriberManager.subscribedCvarValues[i][0] === cvarName) {
                    //Run the callback, providing it with the value of the Cvar it was asking about
                    SubscriberManager.subscribedCvarValues[i][1].resolve(Number(cvarValue));
                    //Remove this subscriber
                    SubscriberManager.subscribedCvarValues.splice(i, 1);
                    lineHandledByCvarListener = true;
                    break;
                }
            }
            if (lineHandledByCvarListener)
                continue;
            for (let i = 0; i <= SubscriberManager.subscribers.length; i++) {
                if (i === SubscriberManager.subscribers.length) {
                    SubscriberManager.log.trace(`No suitable subscriber found for message: ${line}`);
                }
                else {
                    /*
                     * If the subscribed callback returns true, it means that it is responsible for handling
                     * the provided message. If it returns false, we should keep iterating over our subscribers to find
                     * which
                     */
                    const subscriberCanHandleLine = SubscriberManager.subscribers[i].canHandle(line);
                    if (subscriberCanHandleLine) {
                        SubscriberManager.subscriberLog.debug(`Selected listener '${SubscriberManager.subscribers[i]}' to handle line '${line}'.`);
                        //We've found a suitable method to handle the message
                        try {
                            //Can't run 'await SubscriberManager.subscribers[i].handleLine(line);' because this would cause a deadlock
                            SubscriberManager.subscribers[i].handleLine(line).then(() => SubscriberManager.subscriberLog.debug(`Listener '${SubscriberManager.subscribers[i]}' finished handling line '${line}'.`));
                        }
                        catch (e) {
                            SubscriberManager.subscriberLog.error(`Listener '${SubscriberManager.subscribers[i]}' encountered an error handling line '${line}'.`);
                            SubscriberManager.subscriberLog.error(e);
                        }
                        break;
                    }
                }
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) await _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
};
SubscriberManager.patientlyWaitForConsoleSocket = async () => {
    try {
        await SubscriberManager.waitForConsoleSocket(true);
    }
    catch (e) {
        throw e;
    }
};
SubscriberManager.waitForConsoleSocket = async (patient) => {
    const requestId = uuid_1.v4();
    try {
        if (patient) {
            SubscriberManager.log.debug(`(Request ID ${requestId}) Waiting on TCP port ${SubscriberManager.port} for ${SubscriberManager.waitOnOpts.timeout / 1000} seconds.`);
            await waitOn(SubscriberManager.waitOnOpts);
            SubscriberManager.log.debug(`(Request ID ${requestId}) Done waiting!`);
        }
        else {
            SubscriberManager.log.debug(`(Request ID ${requestId}) Waiting on TCP port ${SubscriberManager.port} for ${SubscriberManager.waitOnOptsImpatient.timeout / 1000} seconds.`);
            await waitOn(SubscriberManager.waitOnOptsImpatient);
            SubscriberManager.log.debug(`(Request ID ${requestId}) Done waiting!`);
        }
    }
    catch (err) {
        SubscriberManager.log.error(err);
        SubscriberManager.log.error('Encountered an error when waiting for the console socket to open.');
        throw err;
    }
};
/**
 * Write one or many commands to the console.
 * @param commandOrArray a single string or an array of strings to be written to the console
 */
SubscriberManager.sendMessage = (commandOrArray) => {
    if (Array.isArray(commandOrArray)) {
        for (let command of commandOrArray) {
            SubscriberManager.log.debug(`Writing to CStrike console: '${command}'`);
            SubscriberManager.socket.write(`${command}\n`);
        }
        //TODO: Research if there's a better way to use varargs in TypeScript :)
    }
    else {
        SubscriberManager.log.debug(`Writing to CStrike console: '${commandOrArray}'`);
        SubscriberManager.socket.write(`${commandOrArray}\n`);
    }
};
SubscriberManager.requestCvarValue = (cvarName) => {
    const deferred = pDefer();
    SubscriberManager.subscribedCvarValues.push([cvarName, deferred]);
    SubscriberManager.sendMessage(cvarName);
    SubscriberManager.cvarSubscribersLog.debug(`Added '${cvarName}' to the cvar subscribers list.`);
    return TimeoutPromise_1.TimeoutPromise.timeoutPromise(deferred.promise, `Request for Cvar '${cvarName}'`, false);
};
SubscriberManager.searchForValue = (command, regex, isUserDecision) => {
    const deferred = pDefer();
    SubscriberManager.specialOutputGrabbers.push([regex, deferred]);
    SubscriberManager.sendMessage(command);
    SubscriberManager.valueListenersLog.debug(`Added a value grabber grabbing output from '${command}' to the grabber list.`);
    return TimeoutPromise_1.TimeoutPromise.timeoutPromise(deferred.promise, `Request for response to command '${command}'`, isUserDecision);
};
SubscriberManager.subscribe = (listener) => {
    SubscriberManager.subscribers.push(listener);
    SubscriberManager.subscriberLog.debug(`Added callback '${listener}' to the subscribers list.`);
};
SubscriberManager.unsubscribe = (listener) => {
    const index = SubscriberManager.subscribers.indexOf(listener);
    if (index > -1) {
        SubscriberManager.subscribers.splice(index, 1);
    }
    else {
        SubscriberManager.subscriberLog.warn(`Attempted to unsubscribe callback function '${listener}' but failed.`);
    }
};
//# sourceMappingURL=../../src/utils/SubscriberManager.js.map