"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserDecisionTimeoutException = exports.TimeoutPromise = void 0;
const Config_1 = require("./Config");
class TimeoutPromise {
}
exports.TimeoutPromise = TimeoutPromise;
TimeoutPromise.config = Config_1.Config.getConfig();
TimeoutPromise.consolePromiseTimeoutMs = TimeoutPromise.config.internals.console_output_promise_wait_time * 1000;
TimeoutPromise.consoleUserInputPromiseTimeoutMs = TimeoutPromise.config.internals.console_user_input_wait_time * 1000;
/**
 * @see https://italonascimento.github.io/applying-a-timeout-to-your-promises/
 * @param promise a promise to add a timeout to
 * @param descriptiveTaskName some name that will make sense to someone when they're trying to figure out the specific promise that failed and what it was doing
 * @param isUserDecision whether or not this promise is for a user's decision, in which case the promise should wait console_user_input_wait_time seconds (per config.ini)
 * @returns a promise that will be rejected after config.internals.console_output_promise_wait_time milliseconds but won't be rejected if the provided promise resolves or is rejected before then
 */
TimeoutPromise.timeoutPromise = (promise, descriptiveTaskName, isUserDecision) => {
    // Create a promise that rejects in <ms> milliseconds
    let timeout = new Promise((resolve, reject) => {
        let id = setTimeout(() => {
            clearTimeout(id);
            if (isUserDecision) {
                reject(new UserDecisionTimeoutException(descriptiveTaskName, TimeoutPromise.consoleUserInputPromiseTimeoutMs));
            }
            else {
                reject(`${descriptiveTaskName} timed out in ${TimeoutPromise.consolePromiseTimeoutMs}ms.`);
            }
        }, isUserDecision ? TimeoutPromise.consoleUserInputPromiseTimeoutMs : TimeoutPromise.consolePromiseTimeoutMs);
    });
    // Returns a race between our timeout and the passed in promise
    return Promise.race([
        promise,
        timeout
    ]);
};
class UserDecisionTimeoutException {
    constructor(taskName, timeOut) {
        this.taskName = taskName;
        this.timeOut = timeOut;
    }
}
exports.UserDecisionTimeoutException = UserDecisionTimeoutException;
//# sourceMappingURL=../../src/utils/TimeoutPromise.js.map