"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.VoicePlayerVolume = void 0;
const pDefer = require("p-defer");
const LogHelper_1 = require("./LogHelper");
const SubscriberManager_1 = require("./SubscriberManager");
class VoicePlayerVolume {
}
exports.VoicePlayerVolume = VoicePlayerVolume;
VoicePlayerVolume.log = LogHelper_1.LogHelper.getLogger('VoicePlayerVolume');
VoicePlayerVolume.setVoicePlayerVolumeByName = async (playerName, volume) => {
    let players = await VoicePlayerVolume.getVoicePlayerVolumeValues();
    const player = players.find((value) => value.PlayerName === playerName);
    if (!player) {
        VoicePlayerVolume.log.warn(`Couldn't find player with name '${playerName}' when attempting to set voice volume.`);
    }
    else {
        VoicePlayerVolume.setVoicePlayerVolume(player.PlayerNumber, volume);
    }
};
VoicePlayerVolume.setVoicePlayerVolume = (playerNumber, volume) => {
    SubscriberManager_1.SubscriberManager.sendMessage(`voice_player_volume ${playerNumber} ${volume}`);
};
VoicePlayerVolume.getVoicePlayerVolumeValues = async () => {
    const deferred = pDefer();
    const listener = new VoicePlayerVolumeListener(deferred);
    SubscriberManager_1.SubscriberManager.subscribe(listener);
    SubscriberManager_1.SubscriberManager.sendMessage('voice_player_volume');
    deferred.promise.then(() => SubscriberManager_1.SubscriberManager.unsubscribe(listener));
    return deferred.promise;
};
class VoicePlayerVolumeListener {
    constructor(promise) {
        this.players = [];
        this.reading = false;
        this.promise = promise;
    }
    name() {
        return VoicePlayerVolumeListener.name;
    }
    canHandle(consoleLine) {
        return VoicePlayerVolumeListener.isVoicePlayerVolumePadding(consoleLine) || VoicePlayerVolumeListener.voicePlayerVolumeRegExp.test(consoleLine);
    }
    handleLine(consoleLine) {
        if (!this.reading && VoicePlayerVolumeListener.paddingDashesRegExp.test(consoleLine)) {
            this.reading = true;
        }
        else if (this.reading && VoicePlayerVolumeListener.voicePlayerVolumeRegExp.test(consoleLine)) {
            let playerInfo = VoicePlayerVolumeListener.voicePlayerVolumeRegExp.exec(consoleLine);
            if (playerInfo) {
                const playerVolume = {
                    PlayerNumber: Number(playerInfo[1]),
                    PlayerName: String(playerInfo[2].trim()),
                    Volume: Number(playerInfo[3]),
                };
                if (this.reading) {
                    this.players.push(playerVolume);
                }
            }
            else {
                //Maybe throw error here
                VoicePlayerVolumeListener.log.error('Failed to split output for player volume info using RegExp.exec()');
            }
        }
        else if (this.reading && VoicePlayerVolumeListener.paddingDashesRegExp.test(consoleLine)) {
            this.reading = false;
            this.promise.resolve(this.players);
        }
        return Promise.resolve(undefined);
    }
}
VoicePlayerVolumeListener.whitespaceRegExp = /\s{2,}/g;
VoicePlayerVolumeListener.paddingDashesRegExp = RegExp('-{2,}');
VoicePlayerVolumeListener.voicePlayerVolumeRegExp = RegExp('(\\d+)\ +(.*)\ +(\\d\\.\\d{2})');
VoicePlayerVolumeListener.log = LogHelper_1.LogHelper.getLogger('VoicePlayerVolumeListener');
VoicePlayerVolumeListener.isVoicePlayerVolumePadding = (text) => {
    const processedLine = text.replace(VoicePlayerVolumeListener.whitespaceRegExp, ' ').trim();
    const splitText = processedLine.split(' ');
    let parsedLineIsPaddingCharacters = true;
    for (let element of splitText) {
        if (!VoicePlayerVolumeListener.paddingDashesRegExp.test(element)) {
            parsedLineIsPaddingCharacters = false;
            break;
        }
    }
    return parsedLineIsPaddingCharacters;
};
//# sourceMappingURL=../../src/utils/VoicePlayerVolume.js.map